@@ -6,12 +6,12 @@ from dask import delayed
 import pytest
 from tornado import gen
 
-from distributed import Worker
+from distributed import Worker, Client
 from distributed.client import wait
 from distributed.utils import tokey
-from distributed.utils_test import (inc, gen_cluster,
+from distributed.utils_test import (inc, gen_cluster, cluster,
                                     slowinc, slowadd)
-from distributed.utils_test import loop # flake8: noqa
+from distributed.utils_test import loop # noqa: F401
 
 
 @gen_cluster(client=True, ncores=[])
@@ -50,7 +50,7 @@ def test_resource_submit(c, s, a, b):
     yield wait(y)
     assert y.key in b.data
 
-    assert z.key in s.unrunnable
+    assert s.get_task_status(keys=[z.key]) == {z.key: 'no-worker'}
 
     d = Worker(s.ip, s.port, loop=s.loop, resources={'C': 10})
     yield d._start()
@@ -118,7 +118,7 @@ def test_persist(c, s, a, b):
 
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1, {'resources': {'A': 1}}),
-                                  ('127.0.0.1', 1, {'resources': {'B': 1}})])
+                                  ('127.0.0.1', 1, {'resources': {'B': 11}})])
 def test_compute(c, s, a, b):
     x = delayed(inc)(1)
     y = delayed(inc)(x)
@@ -128,6 +128,12 @@ def test_compute(c, s, a, b):
 
     assert b.data
 
+    xs = [delayed(inc)(i) for i in range(10, 20)]
+    xxs = c.compute(xs, resources={'B': 1})
+    yield wait(xxs)
+
+    assert len(b.data) > 10
+
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1, {'resources': {'A': 1}}),
                                   ('127.0.0.1', 1, {'resources': {'B': 1}})])
@@ -188,11 +194,14 @@ def test_prefer_constrained(c, s, a):
     start = time()
     yield wait(constrained)
     end = time()
-    assert end - start < 1
+    assert end - start < 4
+    has_what = dict(s.has_what)
+    processing = dict(s.processing)
+    assert len(has_what) < len(constrained) + 2  # at most two slowinc's finished
     assert s.processing[a.address]
 
 
-@pytest.mark.xfail(reason="")
+@pytest.mark.skip(reason="")
 @gen_cluster(client=True, ncores=[('127.0.0.1', 2, {'resources': {'A': 1}}),
                                   ('127.0.0.1', 2, {'resources': {'A': 1}})])
 def test_balance_resources(c, s, a, b):
@@ -230,14 +239,14 @@ def test_persist_collections(c, s, a, b):
     z = y.map_blocks(lambda x: 2 * x)
     w = z.sum()
 
-    ww, yy = c.persist([w, y], resources={tuple(y._keys()): {'A': 1}})
+    ww, yy = c.persist([w, y], resources={tuple(y.__dask_keys__()): {'A': 1}})
 
     yield wait([ww, yy])
 
-    assert all(tokey(key) in a.data for key in y._keys())
+    assert all(tokey(key) in a.data for key in y.__dask_keys__())
 
 
-@pytest.mark.xfail(reason="Should protect resource keys from optimization")
+@pytest.mark.skip(reason="Should protect resource keys from optimization")
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1, {'resources': {'A': 1}}),
                                   ('127.0.0.1', 1, {'resources': {'B': 1}})])
 def test_dont_optimize_out(c, s, a, b):
@@ -247,7 +256,46 @@ def test_dont_optimize_out(c, s, a, b):
     z = y.map_blocks(lambda x: 2 * x)
     w = z.sum()
 
-    yield c.compute(w, resources={tuple(y._keys()): {'A': 1}},)
+    yield c.compute(w, resources={tuple(y.__dask_keys__()): {'A': 1}},)
 
-    for key in map(tokey, y._keys()):
+    for key in map(tokey, y.__dask_keys__()):
         assert 'executing' in str(a.story(key))
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1, {'resources': {'A': 1}}),
+                                  ('127.0.0.1', 1, {'resources': {'B': 1}})])
+def test_full_collections(c, s, a, b):
+    dd = pytest.importorskip('dask.dataframe')
+    df = dd.demo.make_timeseries(freq='60s', partition_freq='1d',
+            start='2000-01-01', end='2000-01-31')
+    z = df.x + df.y  # some extra nodes in the graph
+
+    yield c.compute(z, resources={tuple(z.dask): {'A': 1}})
+    assert a.log
+    assert not b.log
+
+
+@pytest.mark.parametrize('optimize_graph', [
+    pytest.mark.xfail(True, reason="don't track resources through optimization"),
+    False
+])
+def test_collections_get(loop, optimize_graph):
+    da = pytest.importorskip('dask.array')
+    with cluster() as (s, [a, b]):
+        with Client(s['address'], loop=loop) as c:
+            def f(dask_worker):
+                dask_worker.set_resources(**{'A': 1})
+
+            c.run(f, workers=[a['address']])
+
+            x = da.random.random(100, chunks=(10,)) + 1
+
+            x.compute(resources={tuple(x.dask): {'A': 1}},
+                      optimize_graph=optimize_graph)
+
+            def g(dask_worker):
+                return len(dask_worker.log)
+
+            logs = c.run(g)
+            assert logs[a['address']]
+            assert not logs[b['address']]
